@@ -77,6 +77,7 @@
     double         gamma;
     double         xbold_factor;
     double         ybold_factor;
+    double         radius;
     double         slant;
 
     int            debug;
@@ -94,7 +95,7 @@
     int            fw_index;
 
   } status = { RENDER_MODE_ALL, FT_ENCODING_NONE, 72, 48, -1,
-               1.0, 0.04, 0.04, 0.22,
+               1.0, 0.04, 0.04, 0.02, 0.22,
                0, 0, 0, 0, 0, NULL, { 0 }, 0, 0,
                0, { 0x10, 0x40, 0x70, 0x40, 0x10 }, 2 };
 
@@ -124,10 +125,13 @@
   Render_Stroke( int  num_indices,
                  int  first_index )
   {
-    int         start_x, start_y, step_y, x, y;
-    int         i;
-    FT_Size     size;
-    FT_Stroker  stroker = NULL;
+    int           start_x, start_y, step_y, x, y;
+    int           i;
+    FT_Size       size;
+    FT_Face       face;
+    FT_GlyphSlot  slot;
+
+    FT_Fixed  radius;
 
 
     error = FTDemo_Get_Size( handle, &size );
@@ -139,22 +143,19 @@
     }
 
     INIT_SIZE( size, start_x, start_y, step_y, x, y );
+    face = size->face;
+    slot = face->glyph;
 
-    i = first_index;
+    radius = status.radius * ( status.ptsize * status.res / 72 );
 
-    error = FT_Stroker_New( handle->library, &stroker );
-    if ( error )
-      goto Exit;
-
-    FT_Stroker_Set( stroker, 64,
+    FT_Stroker_Set( handle->stroker, radius,
                     FT_STROKER_LINECAP_ROUND,
                     FT_STROKER_LINEJOIN_ROUND,
                     0 );
 
-    while ( i < num_indices )
+    for ( i = first_index; i < num_indices; i++ )
     {
-      int           gindex;
-      FT_GlyphSlot  slot;
+      int  gindex;
 
 
       if ( handle->encoding == FT_ENCODING_NONE )
@@ -162,9 +163,8 @@
       else
         gindex = FTDemo_Get_Index( handle, i );
 
-      error = FT_Load_Glyph( size->face, gindex,
+      error = FT_Load_Glyph( face, gindex,
                              handle->load_flags | FT_LOAD_NO_BITMAP );
-      slot = size->face->glyph;
 
       if ( !error && slot->format == FT_GLYPH_FORMAT_OUTLINE )
       {
@@ -175,7 +175,7 @@
         if ( error )
           goto Next;
 
-        error = FT_Glyph_Stroke( &glyph, stroker, 1 );
+        error = FT_Glyph_Stroke( &glyph, handle->stroker, 1 );
         if ( error )
         {
           FT_Done_Glyph( glyph );
@@ -183,10 +183,11 @@
         }
 
         error = FTDemo_Draw_Glyph( handle, display, glyph, &x, &y );
-        FT_Done_Glyph( glyph );
+        if ( !error )
+          FT_Done_Glyph( glyph );
 
         if ( error )
-          status.Fail++;
+          goto Next;
         else if ( X_TOO_LONG( x, size, display ) )
         {
           x  = start_x;
@@ -197,18 +198,10 @@
         }
       }
       else
-      {
     Next:
         status.Fail++;
-      }
-
-      i++;
     }
 
-  Exit:
-    if ( stroker )
-      FT_Stroker_Done( stroker );
-
     return error;
   }
 
@@ -217,9 +210,13 @@
   Render_Slanted( int  num_indices,
                   int  first_index )
   {
-    int      start_x, start_y, step_y, x, y;
-    int      i;
-    FT_Size  size;
+    int           start_x, start_y, step_y, x, y;
+    int           i;
+    FT_Size       size;
+    FT_Face       face;
+    FT_GlyphSlot  slot;
+
+    FT_Matrix  shear;
 
 
     error = FTDemo_Get_Size( handle, &size );
@@ -231,13 +228,31 @@
     }
 
     INIT_SIZE( size, start_x, start_y, step_y, x, y );
+    face = size->face;
+    slot = face->glyph;
 
-    i = first_index;
+    /***************************************************************/
+    /*                                                             */
+    /*  2*2 affine transformation matrix, 16.16 fixed float format */
+    /*                                                             */
+    /*  Shear matrix:                                              */
+    /*                                                             */
+    /*         | x' |     | 1  k |   | x |          x' = x + ky    */
+    /*         |    |  =  |      | * |   |   <==>                  */
+    /*         | y' |     | 0  1 |   | y |          y' = y         */
+    /*                                                             */
+    /*        outline'     shear    outline                        */
+    /*                                                             */
+    /***************************************************************/
 
-    while ( i < num_indices )
+    shear.xx = 1 << 16;
+    shear.xy = (FT_Fixed)( status.slant * ( 1 << 16 ) );
+    shear.yx = 0;
+    shear.yy = 1 << 16;
+
+    for ( i = first_index; i < num_indices; i++ )
     {
-      int      gindex;
-      FT_Face  face = size->face;
+      int  gindex;
 
 
       if ( handle->encoding == FT_ENCODING_NONE )
@@ -248,37 +263,12 @@
       error = FT_Load_Glyph( face, gindex, handle->load_flags );
       if ( !error )
       {
-        FT_Matrix    shear;
-        FT_Outline*  outline;
+        FT_Outline_Transform( &slot->outline, &shear );
 
-
-        /***************************************************************/
-        /*                                                             */
-        /*  2*2 affine transformation matrix, 16.16 fixed float format */
-        /*                                                             */
-        /*  Shear matrix:                                              */
-        /*                                                             */
-        /*         | x' |     | 1  k |   | x |          x' = x + ky    */
-        /*         |    |  =  |      | * |   |   <==>                  */
-        /*         | y' |     | 0  1 |   | y |          y' = y         */
-        /*                                                             */
-        /*        outline'     shear    outline                        */
-        /*                                                             */
-        /***************************************************************/
-
-        shear.xx = 1 << 16;
-        shear.xy = (FT_Fixed)( status.slant * ( 1 << 16 ) );
-        shear.yx = 0;
-        shear.yy = 1 << 16;
-
-        outline = &(face->glyph)->outline;
-
-        FT_Outline_Transform( outline, &shear );
-
-        error = FTDemo_Draw_Slot( handle, display, face->glyph, &x, &y );
+        error = FTDemo_Draw_Slot( handle, display, slot, &x, &y );
 
         if ( error )
-          status.Fail++;
+          goto Next;
         else if ( X_TOO_LONG( x, size, display ) )
         {
           x  = start_x;
@@ -289,9 +279,8 @@
         }
       }
       else
+    Next:
         status.Fail++;
-
-      i++;
     }
 
     return error;
@@ -302,9 +291,13 @@
   Render_Embolden( int  num_indices,
                    int  first_index )
   {
-    int      start_x, start_y, step_y, x, y;
-    int      i;
-    FT_Size  size;
+    int           start_x, start_y, step_y, x, y;
+    int           i;
+    FT_Size       size;
+    FT_Face       face;
+    FT_GlyphSlot  slot;
+
+    FT_Pos  xstr, ystr;
 
 
     error = FTDemo_Get_Size( handle, &size );
@@ -316,13 +309,16 @@
     }
 
     INIT_SIZE( size, start_x, start_y, step_y, x, y );
+    face = size->face;
+    slot = face->glyph;
 
-    i = first_index;
+    ystr = status.ptsize * status.res / 72;
+    xstr = status.xbold_factor * ystr;
+    ystr = status.ybold_factor * ystr;
 
-    while ( i < num_indices )
+    for ( i = first_index; i < num_indices; i++ )
     {
-      int      gindex;
-      FT_Face  face = size->face;
+      int  gindex;
 
 
       if ( handle->encoding == FT_ENCODING_NONE )
@@ -336,20 +332,6 @@
         /* this is essentially the code of function */
         /* `FT_GlyphSlot_Embolden'                  */
 
-        FT_GlyphSlot  slot    = face->glyph;
-        FT_Library    library = slot->library;
-        FT_Pos        xstr, ystr;
-
-
-        if ( slot->format != FT_GLYPH_FORMAT_OUTLINE &&
-             slot->format != FT_GLYPH_FORMAT_BITMAP )
-          goto Next;
-
-        ystr = FT_MulFix( face->units_per_EM,
-                          face->size->metrics.y_scale );
-        xstr = (FT_Fixed)( ystr * status.xbold_factor );
-        ystr = (FT_Fixed)( ystr * status.ybold_factor );
-
         if ( slot->format == FT_GLYPH_FORMAT_OUTLINE )
         {
           error = FT_Outline_EmboldenXY( &slot->outline, xstr, ystr );
@@ -365,10 +347,12 @@
           if ( error )
             goto Next;
 
-          error = FT_Bitmap_Embolden( library, &slot->bitmap, xstr, ystr );
+          error = FT_Bitmap_Embolden( slot->library, &slot->bitmap,
+                                      xstr, ystr );
           if ( error )
             goto Next;
-        }
+        } else
+          goto Next;
 
         if ( slot->advance.x )
           slot->advance.x += xstr;
@@ -400,8 +384,6 @@
       else
     Next:
         status.Fail++;
-
-      i++;
     }
 
     return error;
@@ -427,9 +409,7 @@
 
     INIT_SIZE( size, start_x, start_y, step_y, x, y );
 
-    i = first_index;
-
-    while ( i < num_indices )
+    for ( i = first_index; i < num_indices; i++ )
     {
       int  gindex;
 
@@ -450,8 +430,6 @@
         if ( Y_TOO_LONG( y, size, display ) )
           break;
       }
-
-      i++;
     }
 
     return FT_Err_Ok;
@@ -463,15 +441,12 @@
                int  first_index )
   {
     int      start_x, start_y, step_y, x, y;
-    int      i;
     FT_Size  size;
 
     const char*  p;
     const char*  pEnd;
 
 
-    num_indices = num_indices;  /* pacify compiler */
-
     error = FTDemo_Get_Size( handle, &size );
     if ( error )
     {
@@ -481,18 +456,13 @@
 
     INIT_SIZE( size, start_x, start_y, step_y, x, y );
 
-    i = first_index;
-
     p    = (const char*)Text;
     pEnd = p + strlen( (const char*)Text );
 
-    while ( i > 0 )
-    {
+    while ( first_index-- )
       utf8_next( &p, pEnd );
-      i--;
-    }
 
-    while ( num_indices != 0 )
+    while ( num_indices-- )
     {
       FT_UInt  gindex;
       int      ch;
@@ -521,9 +491,6 @@
             break;
         }
       }
-
-      if ( num_indices > 0 )
-        num_indices -= 1;
     }
 
     return FT_Err_Ok;
@@ -568,12 +535,6 @@
 
     for ( pt_size = first_size; pt_size < max_size; pt_size += 64 )
     {
-      sprintf( (char*)text,
-               "%g: the quick brown fox jumps over the lazy dog"
-               " ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789", pt_size / 64.0 );
-
-      p = text;
-
       FTDemo_Set_Current_Charsize( handle, pt_size, status.res );
 
       error = FTDemo_Get_Size( handle, &size );
@@ -593,7 +554,11 @@
       if ( y >= display->bitmap->rows )
         break;
 
-      while ( *p )
+      sprintf( (char*)text,
+               "%g: the quick brown fox jumps over the lazy dog"
+               " ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789", pt_size / 64.0 );
+
+      for ( p = text; *p; p++ )
       {
         FT_UInt  gindex;
 
@@ -605,8 +570,6 @@
           status.Fail++;
         else if ( X_TOO_LONG( x, size, display ) )
           break;
-
-        p++;
       }
     }
 
@@ -659,6 +622,7 @@
     grLn();
     grWriteln( "  x, X        adjust horizontal emboldening" );
     grWriteln( "  y, Y        adjust vertical emboldening" );
+    grWriteln( "  r, R        adjust stroking radius" );
     grWriteln( "  s, S        adjust slanting" );
     grLn();
     grWriteln( "  F           toggle custom LCD filter mode" );
@@ -710,9 +674,9 @@
     grWriteCellString( display->bitmap, 0, 0, "Gamma grid",
                        display->fore_color );
 
-    for ( g = 1; g <= gammas; g += 1 )
+    for ( g = 1; g <= gammas; g++ )
     {
-      double  ggamma = g / 10.0;
+      double  ggamma = 0.1 * g;
       char    temp[6];
       int     y = y_0 + ( yside + 1 ) * ( g - 1 );
       int     nx, ny;
@@ -796,6 +760,23 @@
   }
 
 
+  static void
+  event_radius_change( double  delta )
+  {
+    status.radius += delta;
+
+    if ( status.radius > 0.05 )
+      status.radius = 0.05;
+    else if ( status.radius < 0.0 )
+      status.radius = 0.0;
+
+    sprintf( status.header_buffer, "stroking radius changed to %.3f",
+             status.radius );
+
+    status.header = status.header_buffer;
+  }
+
+
   static void
   event_slant_change( double  delta )
   {
@@ -1034,6 +1015,14 @@
       event_slant_change( -0.02 );
       break;
 
+    case grKEY( 'r' ):
+      event_radius_change( 0.005 );
+      break;
+
+    case grKEY( 'R' ):
+      event_radius_change( -0.005 );
+      break;
+
     case grKEY( 'x' ):
       event_bold_change( 0.005, 0.0 );
       break;
@@ -1436,7 +1425,7 @@
 
     FTDemo_Update_Current_Flags( handle );
 
-    for ( ;; )
+    do
     {
       FTDemo_Display_Clear( display );
 
@@ -1490,9 +1479,7 @@
 
       status.header = 0;
       grListenSurface( display->surface, 0, &event );
-      if ( Process_Event( &event ) )
-        break;
-    }
+    } while ( Process_Event( &event ) == 0 );
 
     printf( "Execution completed successfully.\n" );
     printf( "Fails = %d\n", status.Fail );
